#!/usr/bin/env Rscript
#########################################################
# TASK: Corr plots for PS and Lig 

# Output: 1 svg

#=======================================================================
# working dir and loading libraries
getwd()
setwd("~/git/LSHTM_analysis/scripts/plotting/")
getwd()

#source("combining_dfs_plotting.R")
source("my_pairs_panel.R") # with lower panel turned off
source("corr_data.R")

#=======
# output
#=======
# PS
corrplot_ps_df2 = "corr_PS_df2.svg"
plot_corr_ps_df2 =  paste0(plotdir,"/", corrplot_ps_df2)

corrplot_ps_df3 = "corr_PS_df3.svg"
plot_corr_ps_df3 =  paste0(plotdir,"/", corrplot_ps_df3)

# LIG
corrplot_lig_df2 = "corr_LIG_df2.svg"
plot_corr_lig_df2 =  paste0(plotdir,"/", corrplot_lig_df2)

corrplot_lig_df3 = "corr_LIG_df3.svg"
plot_corr_lig_df3 =  paste0(plotdir,"/", corrplot_lig_df3)

####################################################################
#               end of loading libraries and functions                
####################################################################
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Data for plots
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cols_to_drop = c("ASA", "AF_kin")


# PS
corr_ps_df2 = corr_ps_df2[!colnames(corr_ps_df2)%in%cols_to_drop]
corr_ps_df2 = corr_ps_df2[-1]

corr_ps_df3 = corr_ps_df3[!colnames(corr_ps_df3)%in%cols_to_drop]
corr_ps_df3 = corr_ps_df3[-1]


# Lig
corr_lig_df2 = corr_lig_df2[!colnames(corr_lig_df2)%in%cols_to_drop]
corr_lig_df2 = corr_lig_df2[-1]

corr_lig_df3 = corr_lig_df3[!colnames(corr_lig_df3)%in%cols_to_drop]
corr_lig_df3 = corr_lig_df3[-1]

#---------------------------------------
# generate corr PS plot 1: merged_df2
#---------------------------------------
cat("Corr plot PS DUET with coloured dots:", plot_corr_ps_df2)
svg(plot_corr_ps_df2, width = 30, height = 25)

OutPlot_ps_df2 = pairs.panels(corr_ps_df2[1:(length(corr_ps_df2)-2)]
             , method = "spearman" # correlation method
             , hist.col = "grey" ##00AFBB
             , density = T  # show density plots
             , ellipses = F # show correlation ellipses
             , stars = T
             , rug = F
             , breaks = "Sturges"
             , show.points = T
             , bg = c("#f8766d", "#00bfc4")[unclass(factor(corr_ps_df2$duet_outcome))] # can't use colour as duet and foldx are opposite
             , pch = 21 # for bg
             #, pch = 19
             , jitter = T
             , alpha = 1
             #, points(pch = 19, col = c("#f8766d", "#00bfc4"))
             , cex = 1.8
             , cex.axis = 2
             , cex.labels = 3.8
             , cex.cor = 1
             , smooth = F)

print(OutPlot_ps_df2)
dev.off()

#----------------------------------------------
# generate corr PS plot 2: merged_df3
#----------------------------------------------
cat("Corr plot PS DUET with coloured dots:", plot_corr_ps_df3)
svg(plot_corr_ps_df3, width = 30, height = 25)

OutPlot_ps_df3 = pairs.panels(corr_ps_df3[1:(length(corr_ps_df3)-2)]
                 , method = "spearman" # correlation method
                 , hist.col = "grey" ##00AFBB
                 , density = T  # show density plots
                 , ellipses = F # show correlation ellipses
                 , stars = T
                 , rug = F
                 , breaks = "Sturges"
                 , show.points = T
                 , bg = c("#f8766d", "#00bfc4")[unclass(factor(corr_ps_df3$duet_outcome))] # can't use colour as duet and foldx are opposite
                 , pch = 21 # for bg
                 , cex = 3
                 , cex.axis = 1.6
                 , cex.labels = 3.8
                 , cex.cor = 1
                 , smooth = F
                 
)

print(OutPlot_ps_df3)
dev.off()

################################################################################################

#---------------------------------------
# generate corr lig plot 1: merged_df2
#---------------------------------------
cat("Corr plot lig DUET with coloured dots:", plot_corr_lig_df2)
svg(plot_corr_lig_df2, width = 30, height = 25)

OutPlot_lig_df2 = pairs.panels(corr_lig_df2[1:(length(corr_lig_df2)-2)]
                               , method = "spearman" # correlation method
                               , hist.col = "grey" ##00AFBB
                               , density = T  # show density plots
                               , ellipses = F # show correlation elliliges
                               , stars = T
                               , rug = F
                               , breaks = "Sturges"
                               , show.points = T
                               , bg = c("#f8766d", "#00bfc4")[unclass(factor(corr_lig_df2$ligand_outcome))] # can't use colour as duet and foldx are opposite
                               , pch = 21 # for bg
                               #, pch = 19
                               , jitter = T
                               , alpha = 1
                               #, points(pch = 19, col = c("#f8766d", "#00bfc4"))
                               , cex = 1.8
                               , cex.axis = 2
                               , cex.labels = 3.8
                               , cex.cor = 1
                               , smooth = F)

print(OutPlot_lig_df2)
dev.off()

#----------------------------------------------
# generate corr lig plot 2: merged_df3
#----------------------------------------------
cat("Corr plot lig DUET with coloured dots:", plot_corr_lig_df3)
svg(plot_corr_lig_df3, width = 30, height = 25)

OutPlot_lig_df3 = pairs.panels(corr_lig_df3[1:(length(corr_lig_df3)-2)]
                               , method = "spearman" # correlation method
                               , hist.col = "grey" ##00AFBB
                               , density = T  # show density plots
                               , ellipses = F # show correlation elliliges
                               , stars = T
                               , rug = F
                               , breaks = "Sturges"
                               , show.points = T
                               , bg = c("#f8766d", "#00bfc4")[unclass(factor(corr_lig_df3$ligand_outcome))] # can't use colour as duet and foldx are opposite
                               , pch = 21 # for bg
                               , cex = 3
                               , cex.axis = 1.6
                               , cex.labels = 3.8
                               , cex.cor = 1
                               , smooth = F
                               
)

print(OutPlot_lig_df3)
dev.off()

